@@ -57,6 +57,35 @@ def _get_state_cookie(config, state, req_uri):
     return data.decode('utf-8')
 
 
+def _create_oauth2_redirect_response(scope, receive, name, config):
+    _request = Request(scope, receive)
+    _redirect_uri = config.get('redirect_uri')
+    _client = OAuth2Session(
+        client_id=config['client_id'],
+        redirect_uri=_redirect_uri
+    )
+    _auth_url, state = _client.authorization_url(f"{config['idp']}/authorize")
+    response = RedirectResponse(_auth_url, status_code=307)
+    response.set_cookie(
+        key=name, value=_get_state_cookie(config, state, str(_request.url)),
+        secure=True, samesite='none', httponly=True, expires='session',  # path=oauth2_cookie_path
+    )
+    return response
+
+
+def _get_token_cookie(config, token):
+    _cookie_content = {'type': 'token', 'token': token}
+    data = b64encode(json.dumps(_cookie_content).encode('utf-8'))
+    data = _get_signer(config).sign(data)
+    return data.decode('utf-8')
+
+
+def _fetch_grant_token(config, state, code):
+    _client = OAuth2Session(client_id=config['client_id'], state=state, redirect_uri=config.get('redirect_uri'))
+    _token = _client.fetch_token(token_url=f"{config['idp']}/token", code=code, )
+    return _token
+
+
 async def process(config, scope: Scope, receive: Receive, send: Send) -> None:
     cookie_data = {}
     _oauth_cookie_configs = _get_cookie_configs(config)
@@ -76,8 +105,17 @@ async def process(config, scope: Scope, receive: Receive, send: Send) -> None:
                 _cookie_type = _cookie_data.get('type')
                 if _cookie_type is not None:
                     if _cookie_type == 'state':
-                        # its a state cookie > do oauth2 code_grant req
-                        i = 0
+                        _code = Request(scope, receive).query_params.get('code')
+                        _token = _fetch_grant_token(config=oauth2_cookie_config, state=_cookie_data['state'],
+                                                    code=_code)
+                        _token = OAuth2Session(token=_token).access_token
+                        response = RedirectResponse(_cookie_data['request_uri'], status_code=302)
+                        response.set_cookie(
+                            key=oauth2_cookie_name,
+                            value=_get_token_cookie(oauth2_cookie_config, _token),
+                            secure=True, samesite='none', httponly=True, expires='session',  # path=oauth2_cookie_path
+                        )
+                        return response(scope, receive, send)
                     if _cookie_type == 'token':
                         # real cookie w/ auth data
                         cookie_data.update({oauth2_cookie_name: _cookie_data.get('token')})
@@ -86,19 +124,8 @@ async def process(config, scope: Scope, receive: Receive, send: Send) -> None:
                 #     pass
             else:
                 # no cookie > do oauth2 redirect
-                _request = Request(scope, receive)
-                _redirect_uri = f"{_request.base_url}{oauth2_cookie_path}"
-                if oauth2_cookie_config.get('redirect_uri') is not None:
-                    _redirect_uri = oauth2_cookie_config.get('redirect_uri')
-                _client = OAuth2Session(
-                    client_id=oauth2_cookie_config['client_id'],
-                    redirect_uri=_redirect_uri
-                )
-                _auth_url, state = _client.authorization_url(f"{oauth2_cookie_config['idp']}/authorize")
-                response = RedirectResponse(_auth_url, status_code=307)
-                response.set_cookie(
-                    key=oauth2_cookie_name, value=_get_state_cookie(oauth2_cookie_config, state, str(_request.url)),
-                    secure=True, samesite='none', httponly=True, expires='session', # path=oauth2_cookie_path
+                response = _create_oauth2_redirect_response(
+                    scope=scope, receive=receive, name=oauth2_cookie_name, config=oauth2_cookie_config
                 )
                 return response(scope, receive, send)
 
